/* @passwd.c - interrogate the /etc/passwd and /etc/group files
 */
#ifdef Z_PWD_FUNCS
#include <stdio.h>
#include <fcntl.h>
#include <pwd.h>
#include <grp.h>
#include "file.h"
#include "bopen.h"
#ifndef STANDARD
#include "zstand.h"
#endif


#define MAXPASSWDS 200
#define MAXPASSWDLEN 200
#define PASSWDSEP ':'
#define MAXNAME 20
char pwname[MAXNAME+1];
#define MAXPASSWD 30
char pwpasswd[MAXPASSWD+1];
#define MAXCOMMENT 80
char pwcomment[MAXCOMMENT+1];
char pwdir[PATHSIZE+1];
#define MAXSHELL 80
char pwshell[MAXSHELL+1];
struct passwd rtnpw;
char grname[MAXNAME+1];
char grpasswd[MAXPASSWD+1];
#define MAXMEM 80
#define NMAXMEM 10
char dmem[NMAXMEM][MAXMEM+1];
char *grmem[NMAXMEM];
struct group rtngr;
static char *passwd = "/etc/passwd";
static char *passwds[MAXPASSWDS];
static int numpasswds = 0;
static Bool passwdloaded = NO;
static char *group = "/etc/group";
static char *groups[MAXPASSWDS];
static int numgroups = 0;
static Bool grouploaded = NO;

void zsetpwent(void);
void zsetgwent(void);
char *namenum(char *name, char **list, int numlist);
char *numname(Uid num, char **list, int numlist);
int loadlist(char **list, int maxl, char *file);
struct passwd *zgetpwnam(char *name);
struct group *zgetgrnam(char *name);
struct passwd *zgetpwuid(Uid num);
struct group *zgetgrgid(Gid num);
struct passwd *pwbuftostruct(char *buf, struct passwd *st);
struct group *grbuftostruct(char *buf, struct group *st);

#define COLON ':'

void zsetpwent(void)
	{
	if (!passwdloaded)
		{
		rtnpw.pw_name = pwname;
		rtnpw.pw_dir = pwdir;
		rtnpw.pw_shell = pwshell;
		numpasswds = loadlist(passwds, MAXPASSWDS, passwd);
#		ifdef DEB
		errfmt("doloadpasswd: numpasswds %d\n", numpasswds);
#		endif
		passwdloaded = YES;
		}
	}

void zsetgrent(void)
	{
	if (!grouploaded)
		{
		int count;
		rtngr.gr_name = grname;
		rtngr.gr_mem = &grmem[0];
		for (count = 0; count < NMAXMEM; count++)
			grmem[count] = dmem[count];
		numgroups = loadlist(groups, MAXPASSWDS, group);
#		ifdef DEB
		errfmt("doloadgroup: numgroups %d\n", numgroups);
#		endif
		grouploaded = YES;
		}
	}

char *namenum(char *name, char **list, int numlist)
	{
	int count;
	char dummy1[50];
	char buf[MAXPASSWDLEN+1];
	int index;

#ifdef NEVER
	if (isdigit(name[0]))
		{
		return atoi(name);
		}
#endif
	for (count = 0; count < numlist; count++)
		{
#		ifdef DEB
		errfmt("namenum: '%p'\n", list[count]);
#		endif
		cpyfld(list[count], 1, buf, COLON);
		if (!strcmp(buf, name))
			{
#			ifdef DEB
			errfmt("namenum: found: '%p'\n", list[count]);
#			endif
			return list[count];
			}
		}
	return NULL;
	}

char *numname(Uid num, char **list, int numlist)
	{
	int count;
	char tbuf[MAXPASSWDLEN];

#	ifdef DEB
	errfmt("numname: numlist %d\n", numlist);
#	endif
	for (count = 0; count < numlist; count++)
		{
		cpyfld(list[count], 3, tbuf, COLON);
		if (atoi(tbuf) == num)
			return list[count];
		}
	return NULL;
	}

int loadlist(char **list, int maxl, char *file)
	{
	char namebuf[MAXPASSWDLEN+1];
	FD fd;
	int count, numlist = 0;
	char *newbuf;

	if (0 <= (fd = bopen(file, O_RDONLY|O_BUFFER, 0, &filebufmap)))
		{
		while (0 < get(fd, namebuf, MAXPASSWDLEN))
			{
			if ((newbuf = (char *)malloc(strlen(namebuf) + 1)) != NULL)
				{
				strcpy(newbuf, namebuf);
				list[numlist] = newbuf;
				numlist++;
				}
			}
		bclose(fd);
		}
	else
		eprint("cannot open\n");
	return numlist;
	}

struct passwd *zgetpwnam(char *name)
	{
	char *res;

	zsetpwent();
	if ((res = namenum(name, passwds, numpasswds)) != NULL)
		return pwbuftostruct(res, &rtnpw);
	else
		return NULL;
	}

struct group *zgetgrnam(char *name)
	{
	char *res;

	zsetgrent();
	if ((res = namenum(name, groups, numgroups)) != NULL)
		return grbuftostruct(res, &rtngr);
	else
		return NULL;
	}

struct passwd *zgetpwuid(Uid num)
	{
	char *res;

	zsetpwent();
	if ((res = numname(num, passwds, numpasswds)) != NULL)
		return pwbuftostruct(res, &rtnpw);
	else
		return NULL;
	}

struct group *zgetgrgid(Gid num)
	{
	char *res;

	zsetgrent();
	if ((res = numname(num, groups, numgroups)) != NULL)
		return grbuftostruct(res, &rtngr);
	else
		return NULL;
	}


#define PW_NAME 1
#define PW_PASSWD 2
#define PW_UID 3
#define PW_GID 4
#define PW_COMMENT 5
#define PW_DIR 6
#define PW_SHELL 7
struct passwd *pwbuftostruct(char *buf, struct passwd *st)
	{
	char bf[BUFSIZ];

	cpyfld(buf, PW_NAME, st->pw_name, COLON);
	cpyfld(buf, PW_UID, bf, COLON);
	st->pw_uid = atoi(bf);
	cpyfld(buf, PW_GID, bf, COLON);
	st->pw_gid = atoi(bf);
	cpyfld(buf, PW_DIR, st->pw_dir, COLON);
	cpyfld(buf, PW_SHELL, st->pw_shell, COLON);
	return st;
	}

#define GR_NAME 1
#define GR_PASSWD 2
#define GR_GID 3
#define GR_MEM 4
struct group *grbuftostruct(char *buf, struct group *st)
	{
	char bf[BUFSIZ];

	cpyfld(buf, GR_NAME, st->gr_name, COLON);
	cpyfld(buf, GR_GID, bf, COLON);
	st->gr_gid = atoi(bf);
#	ifdef NEVER
	cpyfld(buf, GR_MEM, st->gr_mem[0], COLON);
#	endif
	return st;
	}

#endif
